@@ -1,6 +1,6 @@
 (**************************************************************************)
 (*  bibtex2html - A BibTeX to HTML translator                             *)
-(*  Copyright (C) 1997-2010 Jean-Christophe Fillitre and Claude March   *)
+(*  Copyright (C) 1997-2014 Jean-Christophe Filliâtre and Claude Marché   *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -15,7 +15,7 @@
 (**************************************************************************)
 
 (* This code is an adaptation of a code written by Xavier Leroy in
-   1995-1997, in its own made latex2html translator. See
+   1995-1997, in his own made latex2html translator. See
 
 @inproceedings{Leroy-latex2html,
                author =        "Xavier Leroy",
@@ -107,6 +107,7 @@ def "\\mathfrak" [Print_arg];
 
 def "\\textin" [Print "<sub>"; Print_arg; Print "</sub>"];
 def "\\textsu" [Print "<sup>"; Print_arg; Print "</sup>"];
+def "\\textsuperscript" [Print "<sup>"; Print_arg; Print "</sup>"];
 def "\\textsi" [Print "<i>" ; Print_arg ; Print "</i>"];
 
 (* Basic color support. *)
@@ -184,6 +185,8 @@ def "\\th" [Print "&thorn;"];
 def "\\TH" [Print "&THORN;"];
 def "\\dh" [Print "&eth;"];
 def "\\DH" [Print "&ETH;"];
+def "\\dj" [Print "&#273;"];
+def "\\DJ" [Print "&#272;"];
 def "\\ss" [Print "&szlig;"];
 def "\\'" [Raw_arg(function "e" -> print_s "&eacute;"
                           | "E" -> print_s "&Eacute;"
@@ -541,6 +544,10 @@ def "\\bgroup" []; (* should go into latexscan? *)
 def "\\egroup" []; (* should go into latexscan? *)
 def "\\citename" [];
 
+(* dashes *)
+def "--" [Print "--"];
+def "---" [Print "---"];
+
 ()
 
 (* Unicode entities *)
@@ -554,6 +561,8 @@ let unicode_entities () =
   def "\\not" [Raw_arg (function
     | "\\models" -> print_s "&#8877;"
     | s -> print_s "not "; print_s s)];
+  def "--" [Print "&#x2013;"];
+  def "---" [Print "&#x2014;"];
   ()
 
 let html_entities () =
@@ -631,6 +640,8 @@ let html_entities () =
     | "a" -> print_s "&abar;"
     | "A" -> print_s "&Abar;"
     | s   -> print_s s)];
+  def "--" [Print "&ndash;"];
+  def "---" [Print "&mdash;"];
   ()
 
 (*s Macros for German BibTeX style. *) 
